Context

Struct Context 

pub struct Context<'a> {
    pub default_dict: Arc<dyn Dict + 'a>,
    /* private fields */
}
Context for evaluating an Expr.

An instance of this type is needed to evaluate Generator invocations. This context also provides a mapping from dictionary hashes to word lists.

Fields

default_dict: Arc<dyn Dict + 'a>

Implementations

Source

impl<'a> Context<'a>

pub fn new( generator: impl IntoIterator<Item = Arc<dyn GeneratorFunc>>, dict: impl IntoIterator<Item = Arc<dyn Dict + 'a>>, default_dict: Arc<dyn Dict + 'a>, ) -> Self

pub fn empty() -> Self

Returns a context without any generators.

pub fn with_default_dict(&mut self, default_dict: Arc<dyn Dict + 'a>) -> Self

Returns a context with the specified default Dict.

The dict is added to the lookup table for the returned context. It may or may not be added to the table for the original context, depending whether there are other clones of the context or not (see Arc::make_mut.)

pub fn dict_hash(args: &[&str]) -> Option<[u8; 32]>

pub fn get_generator( &self, name: &str, ) -> Result<Arc<dyn GeneratorFunc>, NotFound>

pub fn get_dict( &self, hash: &Option<[u8; 32]>, ) -> Result<Arc<dyn Dict + 'a>, NotFound>

Source

impl<'a> Context<'a>

pub fn with_dict(dict: Arc<dyn Dict + 'a>) -> Self